/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * Use animation functions to animate 3D boxes on 
 *****************************************************************************/


int duration = 4000;          // animation duration
int triggertime = -duration;  // animation trigger time when mouse is clicked
boolean useLighting = true;  // set false to show strokes, set true to use filled surfaces with lighting


void setup() {
  size(600, 600, P3D); 
  
}


void draw() { 
  
  // animation time
  float t = (millis() - triggertime)/(float)duration;
  if (t > 1.0)
    t = 0.0;
  
  float rotation = sineInOut(t, 0, 2*PI); // use sine based animation function for rotation
  float translation;
  
  background(0);
  
  if (useLighting) {
    directionalLight(0, 0, 255, 1, 1, 0); 
    directionalLight(255, 0, 0, -1, -1, 0);
    ambientLight(0, 52, 0);
    spotLight(255, 255, 255, width/4, height/4, 2000, 0.03, 0.03, -1, PI/16, 2000); 
    noStroke();
    fill(255);
  }
  else {
    noFill(); 
  }
  
  translate(width/2, height/2);
  
  for (int j = 0; j < 5; j++) { 
    for (int i = 0; i < 5; i++) { 
      
      // compute sine based animation function that runs from 0 to 1 back to 0
      if (t < 0.5) {
        translation = sineInOut(t*2.0, 0, 1);
      }
      else {
        translation = sineInOut(1.0 - (t - 0.5)*2, 0, 1);  
      }
          
      if (!useLighting)
        stroke(255 - translation*abs(i-2)*127, 255 - translation*abs(j-2)*127, 255);
      
      pushMatrix();
      
      translate((i - 2)*translation*width*0.2, (j-2)*translation*height*0.2);
      rotateY(rotation);
      box(70);
      
      if (useLighting)
        sphere(40);
        
      popMatrix();
    }
  }
}

void mouseClicked() {
  
  if (mouseButton == LEFT) {
    triggertime = millis();
  }
}
