/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * Demonstrate animation functions
 *****************************************************************************/

int mode = 0;           // animation mode
int duration = 3000;    // animation cycle duration
PVector vstart, vstop;  // start and stop position
PVector cstart, cstop;  // start and stop color
float power = 4.0;      // power for monomial animation function


void setup() {
  size(400,400);
  
  vstart = new PVector(0, height/2);
  vstop = new PVector(width-1, height/2);
  
  cstart = new PVector(255, 100, 0);
  cstop = new PVector(0, 200, 255);
}


void draw() {
  
  background(0);
  
  // draw animation function
  drawFunction();
  
  // animation cycle time
  float t = (millis() % duration)/(float)duration;
  
  
  PVector position = new PVector(width/2,height/2);
  PVector clr = new PVector(255, 255, 255);
  
  if (mode == 0) { // linear function
    position = linear(t, vstart, vstop);
    clr = linear(t, cstart, cstop);
  }
  else if (mode == 1) { // nonlinear function
    position = nonlinear(t, power, vstart, vstop);
    clr = nonlinear(t, power, cstart, cstop);
  }
  else if (mode == 2) { // inverse nonlinear function
    position = nonlinearInverse(t, power, vstart, vstop);
    clr = nonlinearInverse(t, power, cstart, cstop);
  }
  else if (mode == 3) { // nonlinear in-out function
    position = nonlinearInOut(t, power, vstart, vstop);
    clr = nonlinearInOut(t, power, cstart, cstop);
  }
  else if (mode == 4) { // nonlinear in-out function
    position = sineInOut(t, vstart, vstop);
    clr = sineInOut(t, cstart, cstop);
  }
  
  noStroke();
  fill(clr.x, clr.y, clr.z, 200);
  ellipse(position.x, position.y, 40, 40);
  
}


void drawFunction() {
  
  strokeWeight(3);
  stroke(150);
  
  if (mode == 0) { // linear function
    for (int i = 0; i < width; i++) {
      point(i, linear(i/(float)(height-1), height-1, 0));
    }
  }
  else if (mode == 1) { // nonlinear function
    for (int i = 0; i < width; i++) {
      point(i, nonlinear(i/(float)(height-1), power, height-1, 0));
    }
  }
  else if (mode == 2) { // inverse nonlinear function
    for (int i = 0; i < width; i++) {
      point(i, nonlinearInverse(i/(float)(height-1), power, height-1, 0));
    }
  }
  else if (mode == 3) { // nonlinear in-out function
    for (int i = 0; i < width; i++) {
      point(i, nonlinearInOut(i/(float)(height-1), power, height-1, 0));
    }
  }
  else if (mode == 4) { // sine in out function
    for (int i = 0; i < width; i++) {
      point(i, sineInOut(i/(float)(height-1), height-1, 0));
    }
  } 
}


void keyReleased() {
  if (key == '1') {
    mode = 0; 
  }
  else if (key == '2') {
    mode = 1; 
  }
  else if (key == '3') {
    mode = 2; 
  }
  else if (key == '4') {
    mode = 3; 
  }
  else if (key == '5') {
    mode = 4; 
  }
}
